from bleak import BleakClient
import asyncio
import struct
import gradio as gr

characteristic_bat = "2A19"
characteristic_model = "2A24"
characteristic_collection = "da39c931-1d81-48e2-9c68-d0ae4bbd351f"

async def get_battery_status(addr):
    async with BleakClient(addr) as client:
        value = await client.read_gatt_char(characteristic_bat)
    return value

async def get_device_status(addr):
    info = {}
    async with BleakClient(addr) as client:
        # battery
        bat = await client.read_gatt_char(characteristic_bat)
        bat = str(bat[0])
        info['battery'] = bat

        # model number
        model = await client.read_gatt_char(characteristic_model)
        info['model'] = model.decode('utf-8')

        # record status
        status = await client.read_gatt_char(characteristic_collection)
        info['status'] = str(status[0])

    return info

def status_updater():
    import random
    return f"this is a random {random.random()}"

if __name__ == '__main__':
    addr = "F1:A9:8C:25:AD:1E"

    # a = asyncio.run(get_device_status(addr))
    # print(a)

    with gr.Blocks() as demo:
        memo = gr.Text("info will be displayed here")
        demo.load(fn=status_updater, inputs=None, outputs=memo)

    demo.queue().launch()




import gradio as gr
from glob import glob
import os
import shutil
from tqdm import tqdm 
import time
import zipfile
import tempfile
import psutil

def create_zip(filename, file_paths):
    temp_dir = tempfile.gettempdir()
    zip_path = os.path.join(temp_dir, filename)

    with zipfile.ZipFile(zip_path, 'w', zipfile.ZIP_DEFLATED) as zipf:
        for file in file_paths:
            zipf.write(file, arcname=os.path.basename(file))  # Store without full path
    return zip_path

def get_flash_drives():
    flash_drives = []
    for partition in psutil.disk_partitions():
        if "removable" in partition.opts.lower() or "usb" in partition.device.lower():
            flash_drives.append(partition.device)
    new_dropdown = gr.Dropdown(choices=flash_drives, value=flash_drives[0] if len(flash_drives) > 0 else None, allow_custom_value=True)
    return new_dropdown, default_refresh_btn(), gr.Text("1", label="Wristband name", visible=False), gr.Button("Get Files 📂", visible=True), gr.Button("", visible=False)


def get_msense_files(src_path, label):
    if label == "":
        gr.Warning("Wristband name cannot be empty")
        return "", gr.DownloadButton("No file to be downloaded", interactive=False)

    gr.Info("Start file extraction...")
    progress = gr.Progress()

    file_list = glob(os.path.join(src_path, '*.bin'))
    print(file_list)

    uuid_list = glob(os.path.join(src_path, '*.txt'))

    print(uuid_list)
    file_list.extend(uuid_list)

    progress(0, desc=f"Start copying {len(file_list)} files...")

    dst_dir = tempfile.gettempdir()
    dst_files = []

    try:
        counter = 1
        for f in progress.tqdm(file_list, desc="copying data... consider getting a coffee..."):
            dst_path = os.path.join(dst_dir, os.path.basename(f))
            shutil.copy(f, dst_path)
            dst_files.append(dst_path)
            counter += 1
        
        datetime_str = time.strftime("%y%m%d%H%M")
        zip_name = f"{datetime_str}-{label}.zip"
        zip_path = create_zip(zip_name, dst_files)
        gr.Info(f"File ready")
        return f"Successfully extracted {len(file_list)} to {os.path.basename(zip_path)}", gr.DownloadButton(label="🎉Download data", value=zip_path, interactive=True)
    except Exception as e:
        gr.Error(str(e))
        return str(e), gr.DownloadButton("No file to be downloaded", interactive=False)

def file_extractor_interface():
    with gr.Column():
        with gr.Row():
            msense_path = gr.Dropdown(label="📁 MotionSenSE path", allow_custom_value=True)
            refreash_path_btn = gr.Button("🔄 Refresh / Start over")

        label = gr.Text("", label="Wristband name", visible=False)
        extract_btn = gr.Button("Get Files 📂")
        confirm_btn = gr.Button("", visible=False)

        info_panel = gr.Text(label='Status')

    # files = gr.File(label="Extracted zip file")

    download_btn = default_refresh_btn()

    extract_btn.click(prompt_device_name, outputs=[label, confirm_btn, extract_btn])

    label.change(check_label, inputs=label)

    confirm_btn.click(get_msense_files, inputs=[msense_path, label], outputs=[info_panel, download_btn])
    refreash_path_btn.click(get_flash_drives, outputs=[msense_path, download_btn,
                                                       label,
                                                       extract_btn,
                                                       confirm_btn])

def prompt_device_name():
    return gr.Text("", label="Wristband name", visible=True), gr.Button("Confirm name & Start 🪪", visible=True), gr.Button("Get Files 📂", visible=False)

def default_refresh_btn():
    return gr.DownloadButton("No file to be downloaded", interactive=False)

def check_label(label):
    if label == "":
        gr.Warning("Device name cannot be empty")


# %%
import gradio as gr
from glob import glob
import os
import re
import json
import tempfile
from yams.file_extractor import get_flash_drives

device_info = {}

def get_uuid_from_path(target_path):
    file_list = glob(os.path.join(target_path, 'uuid.txt'))

    if len(file_list) > 0:
        with open(file_list[0], 'r') as f:
            txt = f.read()

            mac_pattern = r'([0-9A-Fa-f]{2}[:]){5}([0-9A-Fa-f]{2})'
            match = re.search(mac_pattern, txt)

            try:
                print("found mac addr", match.group(0))
                return match.group(0)
            except Exception as e:
                return str(e)
    else:
        return "No MotionSenSE found. Plug in or change MotionSenSE path?"
    

def save_device_info(output_path):
    global device_info
    output_path = os.path.join(tempfile.gettempdir(), os.path.basename(output_path))
    with open(output_path, "w") as f:
        json.dump(device_info, f, indent=4)
    return gr.DownloadButton(label=f"Download {os.path.basename(output_path)}", value=output_path)


def add_device_info(serial, uuid):
    global device_info
    device_info[serial] = uuid
    return device_info


def reset_device_info():
    global device_info
    device_info = {}
    return device_info

def update_download(output_name):
    global device_info
    return save_device_info(output_name)

def uuid_extractor_interface():
    with gr.Column():
        with gr.Row():
            msense_path = gr.Dropdown(label="📁 MotionSenSE path", allow_custom_value=True)
            refreash_path_btn = gr.Button("🔄 Refresh")
            refreash_path_btn.click(get_flash_drives, outputs=msense_path)
        extract_btn = gr.Button("🔧 Get UUID")

    with gr.Column():
        uuid_field = gr.Text("UUID will be shown here", label="UUID")
        msense_serial = gr.Text("4ANA002", label="Serial number")
        add_btn = gr.Button("📝 Add uuid-serial pair")

    with gr.Row():
        device_info_preview = gr.JSON(device_info, label="Device info")
        json_reset_btn = gr.Button("🚮 Clear device info")

    with gr.Row():
        output_name = gr.Text("device_info.json", label="Output file name")
        save_btn = gr.DownloadButton("💾 Save device info")

    extract_btn.click(get_uuid_from_path, inputs=msense_path, outputs=uuid_field)
    add_btn.click(add_device_info, inputs=[msense_serial, uuid_field], outputs=device_info_preview)
    save_btn.click(save_device_info, inputs=output_name, outputs=save_btn)
    json_reset_btn.click(reset_device_info, outputs=device_info_preview)

    device_info_preview.change(update_download, inputs=output_name, outputs=save_btn)





from yams.bt_scanner import bleak_scan, collection_ctl, device_info
import asyncio

if __name__ == '__main__':
    asyncio.run(bleak_scan("MSense"))
    collection_ctl(device_info.keys(), False)

    print("All devices stop")